
#

use v5.12;

package Chart::Property;
our $VERSION = 'v2.403.9';

use Carp;

1;

__END__

=pod

=head1 NAME

Chart::Property - managing properties of a chart


=head1 COPYRIGHT & LICENSE

Copyright 2022 Herbert Breunung.

This program is free software; you can redistribute it and/or modify it 
under same terms as Perl itself.

=head1 AUTHOR

Herbert Breunung, <lichtkind@cpan.org>

=cut

