
# Chart::Color: read only color holding object 
#               with methods for relation, mixing and transitions

use v5.12;

package Chart::Property::DataType::Color;
our $VERSION = 'v2.403.9';

use base qw(Graphics::Toolkit::Color);

1;
__END__

=pod

moved to L<Graphics::Toolkit::Color>

=cut
